#include "refidadapter.hpp"

CSMWorld::RefIdAdapter::RefIdAdapter() {}

CSMWorld::RefIdAdapter::~RefIdAdapter() {}

CSMWorld::NestedRefIdAdapterBase::NestedRefIdAdapterBase() {}

CSMWorld::NestedRefIdAdapterBase::~NestedRefIdAdapterBase() {}
