#include "consoleextensions.hpp"

#include <components/compiler/extensions.hpp>
#include <components/compiler/opcodes.hpp>

#include <components/interpreter/interpreter.hpp>
#include <components/interpreter/runtime.hpp>
#include <components/interpreter/opcodes.hpp>

namespace MWScript
{
    namespace Console
    {
        void installOpcodes (Interpreter::Interpreter& interpreter)
        {

        }
    }
}
