// Note: This is not a regular source file.

/// \defgroup components Components

/// \namespace ESMS
/// \ingroup components
/// \brief ESM/ESP record store

/// \namespace ESM
/// \ingroup components
/// \brief ESM/ESP records

/// \namespace FileFinder
/// \ingroup components
/// \brief Linux/Windows-path resolving

/// \namespace ToUTF
/// \ingroup components
/// \brief Text encoding

/// \namespace Compiler
/// \ingroup components
/// \brief script compiler

/// \namespace Interpreter
/// \ingroup components
/// \brief script interpreter

// TODO put nif and nifogre in different namespaces (or merge them)
// TODO put other components into namespaces
