// Note: This is not a regular source file.

/// \ingroup apps
/// \defgroup openmw OpenMW

/// \namespace OMW
/// \ingroup openmw
/// \brief Integration of OpenMW-subsystems

/// \namespace MWDialogue
/// \ingroup openmw
/// \brief NPC dialogues

/// \namespace MWMechanics
/// \ingroup openmw
/// \brief Game mechanics and NPC-AI

/// \namespace MWSound
/// \ingroup openmw
/// \brief Sound & music

/// \namespace MWGUI
/// \ingroup openmw
/// \brief HUD and windows

/// \namespace MWRender
/// \ingroup openmw
/// \brief Rendering

/// \namespace MWWorld
/// \ingroup openmw
/// \brief World data

/// \namespace MWClass
/// \ingroup openmw
/// \brief Workaround for non-OOP design of the record system

/// \namespace MWInput
/// \ingroup openmw
/// \brief User input and character controls

/// \namespace MWScript
/// \ingroup openmw
/// \brief MW-specific script extentions and integration of the script system into OpenMW
