#include "nestedtablewrapper.hpp"

CSMWorld::NestedTableWrapperBase::NestedTableWrapperBase()
{}

CSMWorld::NestedTableWrapperBase::~NestedTableWrapperBase()
{}

int CSMWorld::NestedTableWrapperBase::size() const
{
    return -5;
}
