#ifndef CSV_TOOLS_SUBVIEWS_H
#define CSV_TOOLS_SUBVIEWS_H

namespace CSVDoc
{
    class SubViewFactoryManager;
}

namespace CSVTools
{
    void addSubViewFactories (CSVDoc::SubViewFactoryManager& manager);
}

#endif
