#ifndef INTERPRETER_INSTALLOPCODES_H_INCLUDED
#define INTERPRETER_INSTALLOPCODES_H_INCLUDED

namespace Interpreter
{
    class Interpreter;
    
    void installOpcodes (Interpreter& interpreter);
}

#endif
