/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>

#ifndef _Included_ui_activity_GameActivity_getPathToJni
#define _Included_ui_activity_GameActivity_getPathToJni
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     Java_org_libsdl_app_SDLActivity_getPathToJni
 * Method:    getPathToJni
 * Signature: (I)I
 */
JNIEXPORT void JNICALL Java_ui_activity_GameActivity_getPathToJni(JNIEnv *env, jobject obj, jstring prompt);

#ifdef __cplusplus
}
#endif
#endif
