#include "opcodes.hpp"

namespace Compiler
{
    namespace Control
    {
        const char *controls[numberOfControls] =
        {
            "playercontrols", "playerfighting", "playerjumping", "playerlooking", "playermagic",
            "playerviewswitch", "vanitymode"
        };
    }
}
