#ifndef OPENMW_COMPONENTS_WIDGETS_H
#define OPENMW_COMPONENTS_WIDGETS_H

namespace Gui
{

    /// Register all widgets from this component with MyGUI's factory manager.
    void registerAllWidgets();

}

#endif
